Unit Unit1;

{$MODE objfpc}{$H+}

Interface

Uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  BASSMIDI, bass;

Type

  { TForm1 }

  TForm1 = Class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    CheckBox1: TCheckBox;
    lbSoundfont: TLabel;
    lbLyrics: TLabel;
    Label2: TLabel;
    Memo1: TMemo;
    OpenDialog1: TOpenDialog;
    Timer1: TTimer;
    Procedure Button1Click(Sender: TObject);
    Procedure Button2Click(Sender: TObject);
    Procedure Button3Click(Sender: TObject);
    Procedure FormCreate(Sender: TObject);
    Procedure Timer1Timer(Sender: TObject);
  private
    fStream: HSTREAM;
    fSoundfont: HSOUNDFONT;
  public

  End;

Var
  Form1: TForm1;

Implementation

{$R *.lfm}

Function ProcessMarkText(CaptionText, MarkText: String): String;
Var
  StringList: TStringList;
Begin
  Case MarkText[1] Of
    '@': Result := CaptionText; // skip info.
    '\': Result := Copy(MarkText, 2, pred(Length(MarkText))); // clear line.
    '/': Begin // new line
        StringList := TStringList.Create;
        Try
          StringList.Text := CaptionText;
          StringList.Add(Copy(MarkText, 2, pred(Length(MarkText))));
          While (StringList.Count > 3) Do
            StringList.Delete(0);
          Result := Trim(StringList.Text);
        Finally
          StringList.Free;
        End; {try}
      End;
  Else
    Result := CaptionText + MarkText;
  End; {case}
End; {ProcessMarkText}


Procedure LyricSync(Handle: HSYNC; Stream: HSTREAM; data: DWORD; user: Pointer){$IFDEF LINUX} cdecl{$ELSE} stdcall{$ENDIF};
Var
  Mark: BASS_MIDI_MARK;
Begin
  BASS_MIDI_StreamGetMark(Stream, BASS_SYNC_MIDI_LYRIC, data, Mark);
  TLabel(user).Caption := ProcessMarkText(TLabel(user).Caption, Mark.text);
End; {LyricSync}

{--------------------------------------------------------------------------------------------------}

Procedure TextSync(Handle: HSYNC; Stream: HSTREAM; data: DWORD; user: Pointer){$IFDEF LINUX} cdecl{$ELSE} stdcall{$ENDIF};
Var
  Mark: BASS_MIDI_MARK;
Begin
  BASS_MIDI_StreamGetMark(Stream, BASS_MIDI_MARK_TEXT, data, Mark);
  TLabel(user).Caption := ProcessMarkText(TLabel(user).Caption, Mark.text);
End; {TextSync}

{--------------------------------------------------------------------------------------------------}

Procedure EndSync(Handle: HSYNC; Stream: HSTREAM; data: DWORD; user: Pointer){$IFDEF LINUX} cdecl{$ELSE} stdcall{$ENDIF};
Begin
  TLabel(user).Caption := '';
End; {EndSync}

{ TForm1 }

Procedure TForm1.FormCreate(Sender: TObject);
Var
  sf: BASS_MIDI_FONT;
Begin
  // Check if the Library Version is, correkt.
  If (BASS_GetVersion() Shr 16) <> Bassversion Then Begin
    showmessage('Unable to init the Bass Library ver. :' + BASSVERSIONTEXT);
    halt;
  End;

  If (BASS_MIDI_GetVersion() Shr 16) <> Bassversion Then Begin
    showmessage('Unable to init the BassMidi Library ver. :' + BASSVERSIONTEXT);
    halt;
  End;

  If Not Bass_init(-1, 44100, BASS_DEVICE_DMIX, {$IFDEF LINUX}Nil{$ELSE}0{$ENDIF}, Nil) Then Begin
    showmessage('Unable to init the device, Error code :' + inttostr(BASS_ErrorGetCode));
    halt;
  End;

  // Für was auch immer man das braucht
  If (BASS_MIDI_StreamGetFonts(0, PBASS_MIDI_FONT(@sf), 1) >= 1) Then Begin
    fSoundfont := sf.font;
  End;

  // Diese DLL's stehen eh nicht zu Verfügung ..
  // load optional plugins for packed soundfonts (others may be used too)
  //BASS_PluginLoad('bassflac.dll', 0);
  //BASS_PluginLoad('basswv.dll', 0);

  fStream := 0;
End;

Procedure TForm1.Timer1Timer(Sender: TObject);
Var
  FontInfo: BASS_MIDI_FONTINFO;
Begin
  If BASS_ChannelIsActive(fStream) = 0 Then exit;

  label2.caption := format('%0.2f', [BASS_ChannelBytes2Seconds(fStream, BASS_ChannelGetPosition(fStream, BASS_POS_BYTE))]);

  If BASS_MIDI_FontGetInfo(fSoundfont, FontInfo) Then
    lbSoundfont.Caption := 'name: ' + FontInfo.name + LineEnding +
      'loaded: ' + Format('%d / %d', [FontInfo.samload, FontInfo.samsize])
  Else
    lbSoundfont.Caption := 'no soundfont';
End;

Procedure TForm1.Button1Click(Sender: TObject);
Var
  FileName: String;
  bytes: QWORD;
  t: Double;
  mark: BASS_MIDI_MARK;
Begin
  Memo1.Clear;
  // Laden des .mid files
  If fStream <> 0 Then BASS_StreamFree(fStream);

  FileName := 'test.mid';
  fStream := BASS_MIDI_StreamCreateFile(
    false,
    PChar(FileName),
    0,
    0,
    BASS_SAMPLE_LOOP{$IFDEF UNICODE} Or BASS_UNICODE{$ENDIF}, 1);

  If (fStream = 0) Then Begin
    // it ain't a MIDI
    showmessage('Not a valid midi.');
  End
  Else Begin
    Memo1.Lines.Add(BASS_ChannelGetTags(fStream, BASS_TAG_MIDI_TRACK));
    bytes := BASS_ChannelGetLength(fStream, BASS_POS_BYTE);
    t := BASS_ChannelBytes2Seconds(fStream, bytes);
    Memo1.Lines.add(format('Duration: %0.2fs', [t]));
    lbLyrics.caption := '';
    // set lyrics syncs
    If BASS_MIDI_StreamGetMark(fStream, BASS_MIDI_MARK_LYRIC, 0, mark) Then Begin // got lyrics
      BASS_ChannelSetSync(fStream, BASS_SYNC_MIDI_MARK, BASS_MIDI_MARK_LYRIC, @LyricSync, lbLyrics)
    End
    Else If (BASS_MIDI_StreamGetMark(fStream, BASS_MIDI_MARK_TEXT, 20, mark)) Then Begin // got text instead (over 20 of them)
      BASS_ChannelSetSync(fStream, BASS_SYNC_MIDI_MARK, BASS_MIDI_MARK_TEXT, @TextSync, lbLyrics);
    End;
    BASS_ChannelSetSync(fStream, BASS_SYNC_END, 0, @EndSync, lbLyrics);

    If CheckBox1.Checked Then
      BASS_ChannelFlags(fStream, 0, BASS_MIDI_NOFX); // enable FX
    If Not BASS_ChannelPlay(fStream, false) Then Begin
      showmessage('Error, could not play.');
    End;
  End;
End;

Procedure TForm1.Button2Click(Sender: TObject);
Begin
  If BASS_ChannelIsActive(fStream) = 0 Then BASS_ChannelStop(fStream);
  If fStream <> 0 Then BASS_StreamFree(fStream);
  fStream := 0;
End;

Procedure TForm1.Button3Click(Sender: TObject);
Var
  sf: BASS_MIDI_FONT;
Begin
  If OpenDialog1.Execute Then Begin
    sf.font := BASS_MIDI_FontInit(pChar(OpenDialog1.FileName), 0{$IFDEF UNICODE} Or BASS_UNICODE{$ENDIF}); // open new soundfont
    If (sf.font <> 0) Then Begin
      sf.preset := -1; // use all presets
      sf.bank := 0; // use default bank(s)
      BASS_MIDI_FontFree(fSoundfont); // free old soundfont
      BASS_MIDI_StreamSetFonts(0, PBASS_MIDI_FONT(@sf), 1); // set default soundfont
      BASS_MIDI_StreamSetFonts(fStream, PBASS_MIDI_FONT(@sf), 1); // set for current stream too
      fSoundfont := sf.font;
    End; {if}
  End;
End;

End.

